<?php

/*
 * Licensed to Elasticsearch B.V. under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. Elasticsearch B.V. licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

declare(strict_types=1);

namespace Elastic\Apm\Impl;

use Elastic\Apm\Impl\Log\LoggableInterface;
use Elastic\Apm\Impl\Log\LoggableTrait;
use Elastic\Apm\Impl\Log\LogStreamInterface;

/**
 * Code in this file is part of implementation internals and thus it is not covered by the backward compatibility.
 *
 * @internal
 *
 * @template        T of ExecutionSegment
 */
abstract class ContextDataWrapper implements LoggableInterface
{
    use LoggableTrait;

    /**
     * @var ExecutionSegment
     * @phpstan-var T
     */
    protected $owner;

    /**
     * @param ExecutionSegment $owner
     *
     * @phpstan-param   T      $owner
     *
     */
    protected function __construct(ExecutionSegment $owner)
    {
        $this->owner = $owner;
    }

    protected function beforeMutating(): bool
    {
        return $this->owner->beforeMutating();
    }

    protected function tracer(): Tracer
    {
        return $this->owner->containingTransaction()->tracer();
    }

    /**
     * @return string[]
     */
    protected static function propertiesExcludedFromLog(): array
    {
        return ['owner'];
    }

    /** @inheritDoc */
    public function toLog(LogStreamInterface $stream): void
    {
        $this->toLogLoggableTraitImpl($stream, ['ownerId' => $this->owner->getId()]);
    }
}
